const { Reader } = require("@maxmind/geoip2-node");
const fs = require("fs");
import * as path from "path";

let geoReader: any;

const localDbName = "GeoLite2-Country.mmdb";
const remoteDbName = "RemoteGeoLite2-Country.mmdb";

const checkFileExist = (filePath: string) => fs.existsSync(path.join(__dirname, filePath));

const download = async(url: string, dest: string) => {
  if (checkFileExist(dest)) {
    return;
  };

  const https = require("https");
  const file = fs.createWriteStream(path.join(__dirname, dest));
  return new Promise((resolve: any, reject: any) => {
    https.get(url, (response: any) => {
      response.pipe(file);
      file.on("finish", () => {
        file.close();
        resolve();
      });
    }).on("error", (err: any) => {
      fs.unlink(path.join(__dirname, dest));
      reject(err.message);
    });
  });
};

const downloadDb = async (url: string, dbName: string) => {
  if (!url) {
    return;
  }
  await download(url, dbName);
};

const loadReader = async (): Promise<any> => {
  if(geoReader) { return geoReader; }

  let dbName;
  if (process.env.GEO_DB_URL) {
    await downloadDb(process.env.GEO_DB_URL, remoteDbName);
    dbName = remoteDbName;
  } else {
    dbName = localDbName;
  };

  geoReader = await Reader.open(path.join(__dirname, dbName));
  return geoReader;
}

const getCountry = async (ip: string) => await loadReader().then((reader: any) => reader.country(ip));

const handler = async (req: any, res: any) => {
  const forwardedFor = req.headers["x-forwarded-for"];
  const ips = forwardedFor ? forwardedFor.split(",") : [];
  const ip = ips[0] || req.connection.remoteAddress;

  try {
    const geo = await getCountry(ip) as any;
    res.status(200).json({
      lang: geo.country.names.en === "Vietnam" ? "vi" : "en",
      country: geo.country,
      info: { ...{ip} }
    });
  } catch(err) {
    console.error(err);
    res.status(200).json({
      lang: "en",
      country: null,
      info: { ...{ip}, err }
    });
  }
}

export { getCountry, handler };
